using System;
using System.Text;
using System.Text.RegularExpressions;

namespace SmileProxy {

	/// <summary>
	/// HttpHeader ̊Tv̐łB
	/// </summary>
	public class HttpHeader {

		private HttpHeaderItemCollection	_items		= null;
		private string						_method		= "GET";
		//private string						_uri		= "";
		private	string						_version	= "1.0";

		private string						_host		= "";
		private int							_port		= 80;
		private string						_path		= "";
		private string						_rest		= "";

		public HttpHeader() {
			_items = new HttpHeaderItemCollection();
		}

		public HttpHeaderItemCollection Items {
			get {
				return _items;
			}
		}

		public string Method {
			get {
				return _method;
			}
			set {
				value = value.ToUpper();
				if (value != "GET"  &&
					value != "POST" &&
					value != "HEAD" ) {
					return;
				}
				_method = value;
			}
		}

		public string Uri {
			get {
				if (this.Host == "") {
					return "";
				}

				StringBuilder uri = new StringBuilder();
				uri.Append("http://");
				uri.Append(this.Host);
				if (this.Port != 80) {
					uri.Append(":");
					uri.Append(this.Port);
				}
				uri.Append(this.Path);
				uri.Append(this.Rest);

				return uri.ToString();
			}
			set {
				Regex r = new Regex("http://([^/:]+)(:(\\d+))?(/\\S*)");
				Match m = r.Match(value);
				if (!m.Success) {
					return;
				}
				
				this.Host = m.Groups[1].Value;
				if (m.Groups[3].Value == null) {
					this.Port = int.Parse(m.Groups[3].Value);
				} else {
					this.Port = 80;
				}
				this.Path = m.Groups[4].Value;
				this.Rest = m.Groups[5].Value;
			}
		}

		public string Host {
			get {
				return _host;
			}
			set {
				if (value == "") {
					value = "";
				}
				_host = value;
			}
		}

		public int Port {
			get {
				return _port;
			}
			set {
				if (value <= 0 || 65536 < value) {
					return;
				}
				_port = value;
			}
		}

		public string Path {
			get {
				return _path;
			}
			set {
				if (value == null) {
					value = "";
				}
				_path = value;
			}
		}

		public string Rest {
			get {
				return _rest;
			}
			set {
				if (value == "") {
					value = "";
				}
				_rest = value;
			}
		}

		public string Version {
			get {
				return _version;
			}
			set {
				value = value.ToUpper();
				if (value != "0.9" &&
					value != "1.0" &&
					value != "1.1") {
					return;
				}
				_version = value;
			}
		}


		public string GetHeader() {
			StringBuilder head = new StringBuilder();

			if (this.Uri == "") {
				return "";
			}
			head.Append(this.Method);
			head.Append(" ");
			head.Append(this.Path);
			head.Append(this.Rest);
			head.Append(" HTTP/");
			head.Append(this.Version);
			head.Append("\r\n");

			foreach (HttpHeaderItem item in Items) {
				if (item == null || item.Name == "" || item.Value == "") {
					continue;
				}

				head.Append(item.Name);
				head.Append(": ");
				head.Append(item.Value);
				head.Append("\r\n");
			}
			head.Append("\r\n");

			return head.ToString();
		}

		public void SetHeader(string request) {
			request = request.Replace("\r\n","\n");
			request = request.Replace("\r"  ,"\n");
			request = request.Trim();

			int i;
			int n;
			int len;
			string[] tmp;
			string[] lines;
			HttpHeaderItem item = null;

			lines = request.Split('\n');
			len   = lines.Length;
			if (len <= 0) {
				return;
			}

			// --------------------------------------------------------

			tmp = lines[0].Split(' ');
			if (tmp.Length < 3) {
				return;
			}

			this.Method  = tmp[0];
			this.Uri     = tmp[1];

			tmp = tmp[2].Split('/');
			if (tmp.Length < 2) {
				return;
			}
			this.Version = tmp[1];

			// --------------------------------------------------------

			for (i=1;i<len;i++) {
				n = lines[i].IndexOf(":");
				if (n < 0) {
					continue;
				}

				item  = new HttpHeaderItem();
				item.Name  = lines[i].Substring(0,n).Trim();
				item.Value = lines[i].Substring(n+1).Trim();
				_items.Add(item);
			}

			_items["KeepAlive"]        = null;
			_items["Range"]            = null;
			_items["Proxy-Connection"] = null;

			_items.Add(new HttpHeaderItem("Connection","close"));
		}
	}
}
