using System;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using System.Collections;

namespace SmileProxy {
	/// <summary>
	/// CommandListener ̊Tv̐łB
	/// </summary>
	public class Listener {

		private volatile bool	_IsStarted		= false;
		private ArrayList		_WorkerThreads	= null;
		private Thread			_ListenerThread = null;
		private TcpListener		_ListenerSocket = null;
		private IPEndPoint		_LocalEndPoint  = null;

		public bool IsStarted {
			get {
				return _IsStarted;
			}
		}

		public Listener() 
			: this (new IPEndPoint(IPAddress.Any, 9999)) {
		}

		public Listener(int localPort)
			: this (new IPEndPoint(IPAddress.Any, localPort)) {
		}

		public Listener(IPAddress localAddress, int localPort) 
			: this (new IPEndPoint(localAddress, localPort)) {
		}

		public Listener(IPEndPoint localEndPoint) {
			if (localEndPoint == null) {
				throw new ArgumentNullException("localEndPoint is null");
			}
			_LocalEndPoint = localEndPoint;
		}

		public void Start() {
			if (_IsStarted == true) {
				throw new InvalidOperationException("The server is already started");
			}

			_IsStarted = true;

			_WorkerThreads = new ArrayList();

			_ListenerSocket = new TcpListener( _LocalEndPoint );
			_ListenerSocket.Start();

			_ListenerThread = new Thread( new ThreadStart( Run ) );	
			_ListenerThread.Start();
		}

		public void Stop() {
			if (_IsStarted == false) {
				return;
			}

			_IsStarted = false;

			_ListenerSocket.Stop();

			if (_WorkerThreads != null) {
				foreach (Thread thread in _WorkerThreads) {
					if (thread.ThreadState != ThreadState.Stopped) {
						thread.Abort();
					}
				}
				_WorkerThreads = null;
			}
		}

		private void Run() {
			int			cnt		= 0;
			Socket		sock	= null;
			Worker		proc	= null;
			Thread		thread  = null;

			try {
				while (_IsStarted) {
					sock = _ListenerSocket.AcceptSocket();
					cnt++;

					if (cnt % 1000 == 0) {
						_CollectWorker();
					}

					proc = new Worker( sock );

					thread = new Thread( new ThreadStart( proc.Run ) );
					_WorkerThreads.Add( thread );
					thread.Start();
				}
			}
			catch (ThreadAbortException) {
			}
			catch (SocketException) {
			}
		}

		private void _CollectWorker() {
			ArrayList WorkerThreadsNew = null;
			WorkerThreadsNew = new ArrayList();

			foreach (Thread thread in _WorkerThreads) {
				if (thread.ThreadState != ThreadState.Stopped) {
					WorkerThreadsNew.Add( thread );
				}
			}

			_WorkerThreads = WorkerThreadsNew;
		}

	}
}
