using System;
using System.Net;
using System.Net.Sockets;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;

namespace SmileProxy {

	public class Worker {

		private Socket			_ClientSocket	= null;
		private Socket			_ServerSocket	= null;
		private int				_BufferSize		= 8192;
		private HttpHeader		_HttpHeader		= null;

		public Worker(Socket ClientSocket) {
			_ClientSocket = ClientSocket;
		}

		public void Run() {
			if (_ClientSocket == null) {
				return;
			}

			try {
				this.RecvRequest();
				this.SendRequest();
				this.TransferResponse();
			}
			finally {
				if (_ServerSocket != null) {
					_ServerSocket.Shutdown( SocketShutdown.Both );
					_ServerSocket.Close();
					_ServerSocket = null;
				}
				if (_ClientSocket != null) {
					_ClientSocket.Shutdown( SocketShutdown.Both );
					_ClientSocket.Close();
					_ClientSocket = null;
				}
			}

			return;
		}

		public void RecvRequest() {
			NetworkStream	networkStream	= null;
			BufferedStream	bufferedStream	= null;
			BinaryReader	binaryReader	= null;
			string			request			= "";
			byte[]			buffer			= new byte[_BufferSize];
			int				length			= 0;

			try {
				networkStream  = new NetworkStream( _ClientSocket, FileAccess.Read );
				bufferedStream = new BufferedStream( networkStream, _BufferSize );
				binaryReader   = new BinaryReader( bufferedStream );

				while (true) {
					length = binaryReader.Read(buffer,0,buffer.Length);
					if (length <= 0) {
						break;
					}
					request += Encoding.ASCII.GetString(buffer,0,length);

					if (request.EndsWith("\r\n\r\n")) {
						break;
					}
				}
			}
			finally {
				if (binaryReader != null) {
					binaryReader.Close();
					binaryReader = null;
				}
				if (bufferedStream != null) {
					bufferedStream.Close();
					bufferedStream = null;
				}
				if (networkStream != null) {
					networkStream.Close();
					networkStream = null;
				}
			}

			_HttpHeader = new HttpHeader();
			_HttpHeader.SetHeader(request);

		}

		public void SendRequest() {
			if (_HttpHeader.Host == "") {
				return;
			}

			_ServerSocket = new Socket(
				AddressFamily.InterNetwork, 
				SocketType.Stream,
				ProtocolType.Tcp);

			IPAddress  remoteIpAddr   = System.Net.Dns.Resolve(_HttpHeader.Host).AddressList[0];
			IPEndPoint remoteEndPoint = new IPEndPoint(remoteIpAddr,_HttpHeader.Port); 

			_ServerSocket.Connect(remoteEndPoint);

			NetworkStream	networkStream	= null;
			BufferedStream	bufferedStream	= null;
			StreamWriter	sw	= null;

			try {
				networkStream  = new NetworkStream( _ServerSocket, FileAccess.Write );
				bufferedStream = new BufferedStream( networkStream, _BufferSize );

				sw = new StreamWriter(bufferedStream, Encoding.ASCII );
				sw.Write( _HttpHeader.GetHeader() );
			}
			finally {
				if (sw != null) {
					//sw.Flush();
					sw.Close();
					sw = null;
				}
				if (bufferedStream != null) {
					//bufferedStream.Flush();
					bufferedStream.Close();
					bufferedStream = null;
				}
				if (networkStream != null) {
					//networkStream.Flush();
					networkStream.Close();
					networkStream = null;
				}
			}
		}

		public void TransferResponse() {
			if (_ClientSocket == null) {
				return;
			}
			if (_ServerSocket == null) {
				return;
			}

			int length = 0;
			byte[] buffer = new byte[_BufferSize];

			NetworkStream	networkStreamServer		= null;
			BufferedStream	bufferedStreamServer	= null;
			BinaryReader	binaryReaderServer		= null;

			NetworkStream	networkStreamClient		= null;
			BufferedStream	bufferedStreamClient	= null;
			BinaryWriter	binaryWriterClient		= null;

			CachedStream	cachedStream			= null;

			try {
				networkStreamServer  = new NetworkStream( _ServerSocket, FileAccess.Read );
				bufferedStreamServer = new BufferedStream( networkStreamServer, _BufferSize );
				binaryReaderServer   = new BinaryReader( bufferedStreamServer );

				networkStreamClient  = new NetworkStream( _ClientSocket, FileAccess.Write );

				Regex r = new Regex("http://[^/]+(?:smilevideo|nicovideo)\\.jp/smile\\?v=([^.]+)\\.\\d+(low)?");
				Match m = r.Match(_HttpHeader.Uri);
				if (m.Success) {
					cachedStream = new CachedStream("sm" + m.Groups[1].Value + ".flv", networkStreamClient);
					bufferedStreamClient = new BufferedStream( cachedStream, _BufferSize );
				}
				else {
					bufferedStreamClient = new BufferedStream( networkStreamClient, _BufferSize );
				}
				binaryWriterClient   = new BinaryWriter( bufferedStreamClient );

				if (cachedStream != null && cachedStream.CacheExists) {
					binaryWriterClient.Write("HTTP/1.0 200\r\n\r\n");
				}
				else {
					Console.WriteLine(_HttpHeader.Uri);
					int index = 0;
					byte[] ch = new byte[4];
					while (true) {
						ch[index & 3] = binaryReaderServer.ReadByte();
						networkStreamClient.WriteByte(ch[index & 3]);
						if (ch[(index + 1) & 3] == '\r' && 
							ch[(index + 2) & 3] == '\n' && 
							ch[(index + 3) & 3] == '\r' &&
							ch[(index    ) & 3] == '\n') {
							break;
						}
						index++;
					}
				}

				while (true) {
					length = binaryReaderServer.Read(buffer,0,buffer.Length);
					//length = _ServerSocket.Receive(buffer, 0, buffer.Length, SocketFlags.None );
					if (length <= 0) {
						break;
					}
					binaryWriterClient.Write(buffer,0,length);
				}
			}
			finally {
				if (binaryWriterClient != null) {
					//binaryWriterClient.Flush();
					binaryWriterClient.Close();
					binaryWriterClient = null;
				}
				if (bufferedStreamClient != null) {
					//bufferedStreamClient.Flush();
					bufferedStreamClient.Close();
					bufferedStreamClient = null;
				}
				if (cachedStream != null) {
					cachedStream.Close();
					cachedStream = null;
				}
				if (networkStreamClient != null) {
					//networkStreamClient.Flush();
					networkStreamClient.Close();
					networkStreamClient = null;
				}

				if (binaryReaderServer != null) {
					binaryReaderServer.Close();
					binaryReaderServer = null;
				}
				if (bufferedStreamServer != null) {
					bufferedStreamServer.Close();
					bufferedStreamServer = null;
				}
				if (networkStreamServer != null) {
					networkStreamServer.Close();
					networkStreamServer = null;
				}
			}
		}
	}
}
